<?php

namespace SGalinski\TypoScriptReferenceFrontend\Service;

use SGalinski\TypoScriptReferenceFrontend\Domain\Model\Attribute;
use TYPO3\Flow\Annotations as Flow;

/**
 * The Class TsrefRestService contains methods for communication with TypoScript REST API.
 *
 * @package SGalinski\TypoScriptReferenceFrontend\Service
 * @Flow\Scope("singleton")
 */
class TsrefRestService {
	/**
	 * URL of REST API resource.
	 */
	const RESOURCE_URL = '127.0.0.1:8000/api/attributes';

	/**
	 * Default typo3 version of typoScript attributes.
	 * Filtering by this version (very high number) will fetch attributes of current version.
	 */
	const TYPO3_DEFAULT_VERSION = '99999';

	/**
	 * Authentication token
	 */
	const AUTHENTICATION_TOKEN = 'secretToken'; // TODO: Read it from parameter, and delete this constant.

	/**
	 * Makes curl handle, sets some general options and returns the handle.
	 * It also checks if cURL is installed.
	 *
	 * @param string $suffix
	 * @return resource
	 */
	protected function initialiseCurl($suffix = '') {
		// is cURL installed yet?
		if (!function_exists('curl_init')) {
			throw new \RuntimeException('Sorry cURL is not installed. Install cURL.');
		}

		$curlHandle = curl_init();

		// Set URL to download
		curl_setopt($curlHandle, CURLOPT_URL, self::RESOURCE_URL . $suffix);

		// Should cURL return or print out the data? (true = return, false = print)
		curl_setopt($curlHandle, CURLOPT_RETURNTRANSFER, TRUE);

		// Timeout in seconds
		curl_setopt($curlHandle, CURLOPT_TIMEOUT, 10);
		return $curlHandle;
	}

	/**
	 * Fetches attributes from web resource: RESOURCE_URL.
	 *
	 * @param boolean|null $isType - filters by isType
	 * @param bool $namesOnly - get ids and names only of attributes.
	 * @param string $typo3Version - filters by typo3Version
	 * @param int $typo3Group - filters by typo3Group
	 * @return mixed
	 */
	public function getAttributesJson(
		$isType = NULL, $namesOnly = TRUE, $typo3Version = self::TYPO3_DEFAULT_VERSION,
		$typo3Group = Attribute::NORMAL_GROUP
	) {
		$curlHandle = $this->initialiseCurl();

		$headerProperties = [
			'namesonly: ' . ($namesOnly ? '1' : '0'),
			'version: ' . $typo3Version,
			'typo3group: ' . $typo3Group
		];

		if ($isType !== NULL) {
			$headerProperties[] = 'istype: ' . ($isType ? '1' : '0');
		}

		curl_setopt($curlHandle, CURLOPT_HTTPHEADER, $headerProperties);

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Returns an attribute by its ID from RESOURCE_URL.
	 *
	 * @param int $attributeId
	 * @return mixed JSON
	 */
	public function getAttributeByIdJson($attributeId) {
		$curlHandle = $this->initialiseCurl('/' . $attributeId);

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Returns properties of a type (attribute which is a type) by its ID from RESOURCE_URL/ID/properties.
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getPropertiesByParentIdJson($attributeId) {
		$curlHandle = $this->initialiseCurl('/' . $attributeId . '/properties');

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Gets attribute by id.
	 *
	 * @param int $attributeId
	 * @return \stdClass
	 */
	public function getAttributeById($attributeId) {
		$attributeJson = $this->getAttributeByIdJson($attributeId);
		$attributePhp = json_decode($attributeJson);
		return $attributePhp;
	}

	/**
	 * Gets all type attributes by $typo3Version and $typo3Group from RESOURCE_URL
	 *
	 * @param int $attributeId
	 * @return mixed
	 */
	public function getPropertiesByParentId($attributeId) {
		$propertiesJson = $this->getPropertiesByParentIdJson($attributeId);
		$propertiesPhp = json_decode($propertiesJson);
		return $propertiesPhp;
	}

	/**
	 * Gets all type attributes by $typo3Version and $typo3Group from RESOURCE_URL
	 *
	 * @param bool $namesOnly
	 * @param string $typo3Version
	 * @param int $typo3Group
	 * @return mixed
	 */
	public function getTypes(
		$namesOnly = TRUE, $typo3Version = self::TYPO3_DEFAULT_VERSION, $typo3Group = Attribute::NORMAL_GROUP
	) {
		$typesJson = $this->getAttributesJson(TRUE, $namesOnly, $typo3Version, $typo3Group);
		$typesPhp = json_decode($typesJson);
		return $typesPhp;
	}

	/**
	 * Adds new attribute to REST API resource by use of POST method.
	 *
	 * @param Attribute $attribute
	 * @return mixed
	 */
	public function addNewAttribute(Attribute $attribute) {
		$attributeJson = json_encode($attribute->toArray());
		$curlHandle = $this->initialiseCurl();

		curl_setopt($curlHandle, CURLOPT_POST, 1);
		curl_setopt($curlHandle, CURLOPT_HTTPHEADER, [
			'Content-Type: application/json',
			'accesstoken: ' . self::AUTHENTICATION_TOKEN
		]);
		curl_setopt($curlHandle, CURLOPT_POSTFIELDS, $attributeJson);
		curl_setopt($curlHandle, CURLOPT_RETURNTRANSFER, TRUE);

		// Download the given URL, and return output
		$output = curl_exec($curlHandle);

		// Close the cURL resource, and free system resources
		curl_close($curlHandle);

		return $output;
	}

	/**
	 * Returns associative array of all typo3 groups.
	 *
	 * @return array
	 */
	public function getAllTypo3Groups() {
		return [
			Attribute::NORMAL_GROUP => 'Normal group',
			Attribute::USER_GROUP => 'User group',
			Attribute::PAGE_GROUP => 'Page group',
		];
	}
}